
/*
 * $Id: AggrStat.cc,v 1.2 1999/09/07 17:45:58 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ILog.h"
#include "OLog.h"
#include "AggrStat.h"
#include "gadgets.h"


AggrStat::AggrStat() {
	reset();
}

void AggrStat::reset() {
	theCount = 0;
	theSum = theSqSum = 0;
	theMax = theMin = -1;
}

void AggrStat::record(Val v) {

	if (theCount++) {
		theSum += v;
		theSqSum += v*v;

		if (theMax < v)
			theMax = v;
		else
		if (theMin > v)
			theMin = v;
	} else {
		theSum = theMin = theMax = v;
		theSqSum = v*v;
	}
}

AggrStat &AggrStat::operator +=(const AggrStat &s) {
	if (s.theCount) {
		if (theCount) {
			theCount += s.theCount;
			theSum += s.theSum;
			theSqSum += s.theSqSum;
			theMax = Max(theMax, s.theMax);
			theMin = Min(theMin, s.theMin);
		} else {
			theCount = s.theCount;
			theSum = s.theSum;
			theSqSum = s.theSqSum;
			theMax = s.theMax;
			theMin = s.theMin;
		}
	}
	return *this;
}

void AggrStat::store(OLog &log) const {
	//cerr << here << theCount << ' ' << theMin << ' ' << theMax << ' ' << theSum << ' ' << theSqSum << endl;
	log << theCount << theMin << theMax << theSum << theSqSum;
}

void AggrStat::load(ILog &log) {
	log >> theCount >> theMin >> theMax >> theSum >> theSqSum;
	//cerr << here << theCount << ' ' << theMin << ' ' << theMax << ' ' << theSum << ' ' << theSqSum << endl;
}
