
/*
 * $Id: OidCache.cc,v 1.1.2.1 1999/10/13 17:27:51 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Oid.h"
#include "OidCache.h"

OidCache::OidCache(int aCapacity) {
	if (aCapacity > 0)
		capacity(aCapacity);
}

void OidCache::capacity(int aCap) {
	Assert(aCap >= 0);
	Assert(!theHash.count());

	theHistory.resize(aCap);

	// would be nice to find the closest prime
	aCap = 2*aCap | 1;
	Assert(aCap <= 0xFFFF); // we use short int as an index
	theHash.stretch(2*aCap | 1);
	theHash.count(theHash.capacity());
}

bool OidCache::hasOid(const OidRec &oidr) const {
	return theHash[hash(oidr)] > 0;
}

void OidCache::putOid(const OidRec &oidr) {
	if (theHistory.full())
		purgeOne();
	const short int idx = hash(oidr);
	if (theHash[idx] < (HashItem)0xFF)
		theHash[idx]++;
	theHistory.enqueue(idx);
}

int OidCache::hash(const OidRec &oidr) const {
	return (oidr.oid + oidr.sidx) % theHash.capacity();
}

void OidCache::purgeOne() {
	const short int idx = theHistory.dequeue();
	if (theHash[idx] > (HashItem)0)
		theHash[idx]--;
}
