
/*
 * $Id: PolyClt.cc,v 1.10.2.3 1999/10/11 21:09:11 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "AsyncClt.h"
#include "SyncClt.h"
#include "OLog.h"
#include "LogComment.h"
#include "SharedOpts.h"
#include "ServerRep.h"
#include "CltOpts.h"
#include "CltConnMgr.h"
#include "PolyClt.h"
#include "WorkSetFreezer.h"

#include "PglSemx.h"
#include "RobotSym.h"

#include "gadgets.h"
#include "rndDistrs.h"
#include "globals.h"
#include "shutdown.h"


PolyClt::PolyClt() {
	theAgentType = "Robot";
}

Agent *PolyClt::makeAgent(const AgentCfg &cfg) {
	Assert(cfg.sym);
	
	const RobotSym *rcfg = &(const RobotSym&)cfg.sym->cast("Robot");

	double rr;
	Client *client = 0;
	if (rcfg->reqRate(rr))
		client = new AsyncClt();
	else
		client = new SyncClt();

	Assert(client);
	client->configure(rcfg, *cfg.host, agentMustBind);

	return client;
}

void PolyClt::configure() {

	PolyApp::configure();

	// working set freezer
	if (PglSemx::TheWorkSetLen >= 0)
		(void)new WorkSetFreezer(PglSemx::TheWorkSetLen);

	// new oid prefetching
	ServerRep::ConfigureNids(TheCltOpts.theNidsPrefDepth, TheOpts.theNidsPerMsgMax);
}

void PolyClt::getOpts(Array<OptGrp*> &opts) {
	PolyApp::getOpts(opts);
	opts.append(&TheCltOpts);
}

void PolyClt::reportCfg() {
	PolyApp::reportCfg();
}

int main(int argc, char *argv[]) {
	PolyClt app;
	return app.run(argc, argv);
}
