
/*
 * $Id: SrvHistory.cc,v 1.4.2.1 1999/10/11 21:09:13 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Clock.h"
#include "ErrorMgr.h"
#include "LogComment.h"
#include "SrvHistory.h"
#include "SharedOpts.h"

#include "polyErrors.h"

static const int TheNewOidsPerRepMax = 16;


SrvHistory::SrvHistory(): theSecOid(0), theAdvOid(0), theSecTime(TheClock) {
}

void SrvHistory::window(int capacity) {
	theAdvWin.resize(capacity);
}

int SrvHistory::newOid() {
	// if the gap is getting too big
	if (theAdvOid - theSecOid >= theAdvWin.capacity()) {
		const Time delay = TheClock - theSecTime;
		if (ReportError(errUnclaimOid) && delay >= Time::Sec(5))
			Comment(4) << "giving up after at least " << delay << endc;

		// assume that oid got stuck due to an error and "reclaim" it
		const int stuckOid = theSecOid + 1;
		Assert(!requested(stuckOid));
		noteOid(stuckOid);

		// check that we fixed the problem and continue
		Assert(theAdvOid - theSecOid < theAdvWin.capacity());
	}

	++theAdvOid;
	requested(theAdvOid, false);
	return theAdvOid;
}

void SrvHistory::noteOid(int oid) {
	if (oid <= theSecOid || requested(oid)) // nothing new
		return;

	if (oid > theAdvOid) {
		ReportError(errUnsolicOid);
		return;
	}
	
	requested(oid, true);

	// increment secured level if possible
	if (oid == theSecOid + 1) {
		do {
			++theSecOid;
		} while (theSecOid < theAdvOid && requested(theSecOid+1));
		theSecTime = TheClock;
	}
}

int SrvHistory::newOids(int needCnt, int &resCnt) {
	resCnt = 0;
	
	int firstOid = -1;
	while (needCnt > 0 && resCnt < TheOpts.theNidsPerMsgMax) {
		const int oid = newOid();
		if (firstOid < 0)
			firstOid = oid;
		else
			Assert(firstOid + resCnt == oid);
		needCnt--;
		resCnt++;
	}

	return firstOid;
}

