
/*
 * $Id: embed_test.cc,v 1.1 1999/09/27 19:44:23 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

// g++ -o embed_test -I .. -I include -I xstd/include embed_test.cc xstd/libxstd.a 

#include "polygraph.h"

#include <limits.h>

#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

#include "Rnd.h"
#include "RndPermut.h"
#include "BitMask.h"
#include "CmdLine.h"
#include "opts.h"
#include "gadgets.h"


class MyOpts: public OptGrp {
	public:
		MyOpts():
			theHelpOpt(this,      "help",             "list of options"),
			theOutFileName(this,  "out <file>",       "redirect console output", "-"),
			theChoiceSpace(this,  "choice_space <int>","embedded object selection space", 10),
			theEmdedCount(this,   "embed_count <int>","average number of embedded objects per container", 10),
			theChoiceSlide(this,  "choice_slide <int>", "hard to explain", 1),
			theSimLength(this,    "sim_length <int>", "total number of request to simulate", 100000)
			{}

		virtual bool validate() const;

	public:
		HelpOpt theHelpOpt;
		StrOpt theOutFileName;
		IntOpt theChoiceSpace;
		IntOpt theEmdedCount;
		IntOpt theChoiceSlide;
		IntOpt theSimLength;
} TheOpts;

struct Stats {
	int emdedCount; // unique embedded objects
	int contCount;  // unique containers

	Stats() { reset(); }
	void reset() { emdedCount = contCount = 0; }
};

static BitMask TheMask;                 // to count unique oids
static Stats TheStats, TheIntvlStats;

static int TheLastOid = 0;

#define ForStats(action) { TheStats.action; TheIntvlStats.action; }



bool MyOpts::validate() const {
	if (0)
		cerr << "error" << endl;
	else
		return true;
	return false;
}


static
void configureLogs(int prec) {
	if (TheOpts.theOutFileName && strcmp(TheOpts.theOutFileName, "-"))
		cout = cerr = clog = *new ofstream(TheOpts.theOutFileName);

	configureStream(cout, prec);
	configureStream(cerr, prec);
	configureStream(clog, prec);
}

static
void configure() {
	configureLogs(2);

	if (TheOpts.theSimLength > 0) {
		TheMask.resize(TheOpts.theChoiceSlide*TheOpts.theSimLength + TheOpts.theEmdedCount);
	}
}

static
ostream &operator <<(ostream &os, const Stats &s) {
	return os
		<< setw(8) << s.emdedCount
		<< ' ' << setw(8) << s.contCount
		<< ' ' << setw(6) << Percent(s.emdedCount, s.contCount)
		;
}

static
void report() {
	static int repCnt = 0;
	if (!repCnt++) {
		cout << '#'
			<< ' ' << setw(8) << "oids"
			<< ' ' << setw(8) << "embed"
			<< ' ' << setw(8) << "contain"
			<< ' ' << setw(6) << "embed%"

			<< ' ' << setw(8) << "i-embed"
			<< ' ' << setw(8) << "i-contn"
			<< ' ' << setw(6) << "i-emb%"
			<< endl;
	}

	cout << 'i'
		<< ' ' << setw(8) << TheLastOid
		<< ' ' << TheStats
		<< ' ' << TheIntvlStats
		<< endl;
}

static
void step() {
	static RndGen rng;

	TheLastOid++; // new container
	ForStats(contCount++);

	for (int i = TheOpts.theEmdedCount; i > 0; --i) {
		const int offset = TheOpts.theChoiceSlide * TheLastOid;
		const int eoid = rng(offset, offset + TheOpts.theChoiceSpace);
		if (!TheMask.isSet(eoid)) {
			TheMask.beSet(eoid);
			ForStats(emdedCount++);
		}
	}
}

static
void run() {
	const int repCycle = Max(1, TheOpts.theSimLength/33);
	for (int i = 0; i < TheOpts.theSimLength; ++i) {
		step();
		if (i % repCycle == 0 && i) {
			report();
			TheIntvlStats.reset();
			if (i == repCycle)
				TheStats.reset();
		}
	}
}

int main(int argc, char *argv[]) {
	CmdLine cmd;
	cmd.configure(Array<OptGrp*>() << &TheOpts);
	if (!cmd.parse(argc, argv) || !TheOpts.validate())
		return -1;

	configure();
	cmd.report(cout);

	run();

	return 0;
}
