
/*
 * $Id: ilog2dlog.cc,v 1.10 1999/09/07 17:46:38 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include "ILog.h"
#include "DLog.h"
#include "polyLogTags.h"
#include "gadgets.h"

static
bool ilog2dlog(const String &fname, ILog &il, DLog &dl, const String &pname) {
	String curPName = 0;
	bool foundPhase = !pname; // found a priory if nothing to search for

	while (LogEntryPx prefix = il.begEntry()) {

		dl.theProgress = il.progress();

		// handle start/end of a phase
		if (prefix.theTag == lgStatPhaseBeg) {
			il >> curPName;
			foundPhase = foundPhase || curPName == pname;
		} else
		if (prefix.theTag == lgStatPhaseEnd) {
			if (pname && curPName == pname) // end of the specified phase
				break;
			curPName = 0;
		} else
		if (foundPhase) {
			switch (prefix.theTag) {
				case lgStatCycleRec: {
					StatIntvlRec pr;
					pr.load(il);
					dl.noteRec(pr);
					break;
				}
				case lgStatPhaseRec: {
					StatPhaseRec ph;
					ph.load(il);
					dl.noteRec(ph);
					break;
				}
			}
		}

		il.endEntry();
	}

	if (il.stream() && !il.stream()->bad())
		return foundPhase;

	cerr << "error reading log file " << fname << ": " << error(errno) << endl;
	return false;
}


// phase name must be already set in dl
bool ilog2dlog(const String &fname, DLog &dl) {
	ILog il;
	if (fname == "-")
		il.stream("stdin", &cin);
	else
		il.stream(fname, (istream*)new ifstream(fname));

	dl.theSources.append(new String(fname));

	if (ilog2dlog(fname, il, dl, dl.thePhaseName))
		return true;

	cerr << here << "no `" << dl.thePhaseName << "' phase in `" << fname << "'" << endl;
	return false;
}
