
/*
 * $Id: Client.h,v 1.23.2.8 1999/10/18 22:51:25 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_CLIENT_CLASS_H
#define PG_CLIENT_CLASS_H

#include "AlarmClock.h"
#include "Farm.h"
#include "Agent.h"
#include "CltXact.h"
#include "BcastRcver.h"

class SockOpt;
class Connection;
class RndDistr;
class PortMgr;
class CltConnMgr;
class ServerRep;
class RobotSym;
class PopModel;
class OidRec;
class OidCache;

class Client: public Agent, public AlarmUser, public BcastRcver {
	public:
		static ServerRep *GetServerRep(int idx);

	public:
		Client();
		virtual ~Client();

		virtual void configure(const RobotSym *cfg, const NetAddr &aHost, bool bind2iface);
		virtual void start();

		virtual void noteXactDone(CltXact *x);
		virtual void noteEmbedded(CltXact *x, const OidRec &embedOidRec);

	protected:
		static bool cAddServer(const NetAddr &addr);
		static void FreezeSrvWorkSetCap();

		PortMgr *getPortMgr(bool bind2iface);
		int scanServers(const RobotSym *cfg);

		void scheduleLaunch(Time lastLaunch);
		bool launch(CltXact *x);
		bool tryLaunch(CltXact *x = 0);
		CltXact *genXact(const OidRec &oidRec);
		CltXact *genXact();

		void genOid(OidRec &oidRec);
		void genOid2(OidRec &oidRec);

		virtual void noteInfoEvent(BcastChannel *ch, InfoEvent ev);

	protected:
		static ObjFarm<CltXact> TheXacts;
		static Array<PortMgr*> ThePortMgrs;  // port managers for all aliases
		static Array<ServerRep*> TheServers; // all servers known to clients
		static int TheClientCnt;             // global client counter
		static int TheTouchAllSrvCnt;        // clients that touched all srvrs
		static int TheShrdWorkSetCap;        // cap for all servers together

		CltConnMgr *theConnMgr;       // one for all servers
		RndDistr *theOriginSel;       // selects origin hosts
		int theTouchedSrvCnt;         // servers with private or secured oids

		Time theLaunchWin;
		int theWaitXactLmt;    // limit on the number of postponed requests

		OidCache *theCache;    // robot cache simulation
		Array<CltXact*> theLaunchDebts; // postponed requests
		int theCcXactLvl;      // outstanding concurrent xactions level
		int theLaunchCount;    // total number of xaction launches
		int theExtraLaunchLvl; // uncompensated out-of-order launches

		PopModel *thePopModel;     // popularity model to select old oids
		double thePublicRatio;     // how often to request a "public" object
		double theRecurRatio;      // how often to re-visit an old object
		double theEmbedRecurRatio; // how often to re-visit an old embedded obj
		bool genUniqUrls;          // each generated URL must be unique

		Array<int> thePrivOids;       // per-origin private oid levels
		Array<int> thePrivWorkSetCap; // per-origin private work set limits
};

#endif
