
/*
 * $Id: Xaction.h,v 1.24.2.1 1999/10/05 19:59:03 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XACTION_CLASS_H
#define XACTION_CLASS_H

#include "Select.h"
#include "UniqId.h"
#include "Oid.h"

class IOBuf;
class Connection;

class Xaction: public Select::User {
	public:
		Xaction() { reset(); }

		void reset();

		const UniqId &id() const { return theId; }
		const OidRec &oidRec() const { return theOidRec; }
		Connection *conn() { return theConn; }
		bool started() const { return theStartTime > 0; }

		Size repSize() const { return theRepSize; }
		Time queueTime() const { return theEnqueTime > 0 ? theStartTime - theEnqueTime : Time::Sec(0); }
		Time lifeTime() const { return theLifeTime; }

		void oidRec(const OidRec &aRec) { theOidRec = aRec; }

	protected:
		void start(Connection *conn);
		void stop(int errNo);
		void stopStats();

		void printMsg(const IOBuf &buf) const;
		void printMsg(const IOBuf &buf, Size maxSize) const;
		void printMsg(const char *buf, Size maxSize) const;

	protected:
		Connection *theConn;
		UniqId theId;

		Time theEnqueTime;  // put in a queue (optional)
		Time theStartTime;  // started doing something
		Time theLifeTime;   // [start, stop]; set in stop()
		OidRec theOidRec;   // various details about the transfered object
		Size theRepSize;    // reply size
		int theEmbedCount;  // number of embedded object in the reply

		enum { stNone = 0, stSpaceWaiting, stHdrWaiting, stBodyWaiting, stDone } theState;
};

#endif
