
/*
 * $Id: AgentArrIter.cc,v 1.1.2.1 1999/09/30 19:38:42 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"
#include "AgentArrIter.h"

#include "AgentSym.h"
#include "PglNetAddrSym.h"


AgentArrIter::AgentArrIter(): theArr(0) {
}

AgentArrIter::AgentArrIter(Array<AgentSym*> *anArr, const String &anAgentType):
	theAgentType(anAgentType) {
	if (anArr)
		start(anArr);
}

AgentArrIter &AgentArrIter::start(Array<AgentSym*> *anArr) {
	Assert(anArr);
	theArr = anArr;
	theAgentIdx = theHostIdx = 0;
	sync();
	return *this;
}

bool AgentArrIter::eof() const {
	return theAgentIdx >= theArr->count();
}

AgentArrIter::operator void *() const {
	Assert(theArr);
	return !eof() ? (void*)(-1) : 0;
}

AgentArrIter &AgentArrIter::operator ++() {
	theHostIdx++;
	sync();
	return *this;
}

const NetAddr &AgentArrIter::host() {
	Assert(!eof());
	return theHost;
}

AgentSym *AgentArrIter::agent() {
	Assert(!eof());
	return theArr->item(theAgentIdx);
}

// must be called after an index change
void AgentArrIter::sync() {
	while (!eof()) {
		if (theHostIdx < theArr->item(theAgentIdx)->hostCount()) {
			if (!theAgentType || theArr->item(theAgentIdx)->isA(theAgentType))
				break;
			theHostIdx++;
		} else {
			theHostIdx = 0;
			theAgentIdx++;
		}
	}
		
	if (!eof())
		theHost = theArr->item(theAgentIdx)->host(theHostIdx)->val();
	else
		theHost = NetAddr();
}
