
/*
 * $Id: SynSymTbl.h,v 1.3 1999/09/07 17:48:21 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef SYNTAX_SYMBOL_TABLE_H
#define SYNTAX_SYMBOL_TABLE_H

// SynSymTbl is a collection of "semantic symbols" with 
// search and accounting capabilities

#include "Map.h"
#include "SynSym.h"

class SynSymTbl;

// table element
class SynSymTblItem {
	public:
		SynSymTblItem();
		SynSymTblItem(const String &aType, const String &aName);
		virtual ~SynSymTblItem();

		bool used() const { return theUseCnt > 0; }
		bool defined() const { return theSym; }
		SynSymTbl *ctx() { return theCtx; }
		SynSym *sym() { return theSym; }
		const TokenLoc &loc() const { return theLoc; }
		const String &name() const { return theName; }
		const String &type() const { return theType; }

		void use() { theUseCnt++; }
		void ctx(SynSymTbl *aCtx) { theCtx = aCtx; }
		void sym(SynSym *aSym) { theSym = aSym; }
		void loc(const TokenLoc &aLoc) { theLoc = aLoc; }

	protected:
		SynSymTbl *theCtx;  // context
		SynSym *theSym;
		String theName;
		String theType;
		TokenLoc theLoc; // declaration location
		int theUseCnt;
};

class SynSymTbl: public Map<SynSymTblItem*> {
	public:
		SynSymTbl();
		virtual ~SynSymTbl();

		virtual void report(ostream &os, const String &pfx) const;
};

#endif
