
/*
 * $Id: Socket.h,v 1.4.6.2 1999/10/19 00:00:57 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XSTD_SOCKET_CLASS_H
#define XSTD_SOCKET_CLASS_H

#include <sys/types.h>
#include <sys/socket.h>

#include "Time.h"
#include "Size.h"
#include "NetAddr.h"

struct ifconf;
struct ifaliasreq;

// socket option holder
struct SockOpt {
	Time linger;  // linger delay
	int nagle;    // 3-way bool
	/* more to come */

	SockOpt(): nagle(0) {}
};

// socket wrapper
class Socket {
	public:
		static bool IsEnabled;  // bind,connect,listen,read,write calls are enabled
		static int TheMaxLevel; // will not allow Level to go higher
		static int Level() { return TheLevel; }

	public:
		Socket(): theFD(-1) {}
		Socket(int aFD): theFD(aFD) {}
		~Socket() {}

		bool create();
		bool create(int domain, int type, int protocol);
		bool connect(const NetAddr &addr);
		bool bind(const NetAddr &addr);
		bool listen();
		Socket accept(struct sockaddr *addr, socklen_t *addr_len);
		Size read(void *buf, Size sz);
		Size write(const void *buf, Size sz);
		bool close();

		bool reuseAddr(bool set);
		bool reusePort(bool set);
		bool linger(Time delay);
		bool blocking(bool set);
		bool tcpDelay(bool set);
		bool sendBuf(Size size);
		bool recvBuf(Size size);

		/* ioctl wrappers */
		bool getIfConf(struct ifconf &cfg);
		bool getIfAddr(const char *ifname, struct sockaddr &addr);
		bool getIfBcastAddr(const char *ifname, struct sockaddr &addr);
		bool addIfAddr(const ifaliasreq &req);
		bool delIfAddr(const ifaliasreq &req);

		bool configure(const SockOpt &opt);

		int fd() const { return theFD; }
		bool isOpen() const { return theFD >= 0; }
		bool isBlocking() const;
		int flags() const;
		int lport() const;
		int rport() const;
		Time rcvTimeout() const;
		Size sendBuf() const;
		Size recvBuf() const;
		int errNo() const;
		bool peerAddr(struct sockaddr *addr, socklen_t &len) const;
		NetAddr peerAddr() const;

		operator void*() const { return theFD >= 0 ? (void*)-1 : (void*)0; }

	protected:
		bool noteNewFD();
		bool setOpt(int optName, bool set);
		bool setOpt(int optName, const void *optVal, socklen_t optLen);
		bool getOpt(int optName, void *optVal, socklen_t *optLen) const;

	protected:
		int theFD;

		static int TheLevel; // number of open sockets
};

#endif
